from datetime import datetime
from pathlib import Path
from pydantic import BaseModel, field_serializer, field_validator
from semver import Version
from typing import Literal

class NoteType(BaseModel):
    id: datetime
    name: str
    fields: list[str]

class CardType(BaseModel):
    name: str
    template: Path

class ImportApkg(BaseModel):
    note_type: str

class ImportCsv(BaseModel):
    content_version: datetime
    delimiter: str
    note_type: str
    file_patterns: list[str]|None
    deck_name_pattern: str
    fields_mapping: list[str]
    fields_static: dict[str, str]
    tags: list[str]

class ImportImages(BaseModel):
    content_version: datetime
    note_type: str
    deck_name_pattern: str
    fields_mapping: dict[str, str]

class TemplateSpec(BaseModel):
    template_version: datetime
    note_type: NoteType
    card_types: list[CardType]
    resource_paths: list[Path]

class ApkgSpec(BaseModel):
    # semver of the content version, appended to the filename after a dash
    content_version: str
    templates: list[str]
    content: list[
        dict[Literal['import_apkg'], ImportApkg]|
        dict[Literal['import_csv'], ImportCsv]|
        dict[Literal['import_images'], ImportImages]
    ]
    resource_paths: list[Path]

    @field_serializer('content_version')
    def dump_content_version(self, ver: Version):
        return str(ver)

    @field_validator('content_version')
    def validate_content_version(cls, version):
        Version.parse(version)
        return version
